/*-
 * Copyright (c) 2004 Jacques A. Vidrine
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#ifndef CELABO_20040211_PROCESSORS_HH
#define CELABO_20040211_PROCESSORS_HH
namespace vuxml {


class AllMatcher : public EntryProcessor {
public:
  AllMatcher(EntryProcessor &proc);
  virtual bool operator()(const Entry &entry);
  virtual void start();
  virtual void end();
private:
  EntryProcessor		&proc_;
};


class VersionMatcher : public EntryProcessor {
public:
  VersionMatcher(std::vector<std::string> &args, EntryProcessor &proc);
  virtual bool operator()(const Entry &entry);
  virtual void start();
  virtual void end();
private:
  std::vector<std::string>	&args_;
  EntryProcessor		&proc_;
};


class TextWriter : public EntryProcessor {
public:
  TextWriter(std::ostream &os);
  virtual bool operator()(const Entry &entry);
private:
  std::ostream	&os_;
};


class VuXMLWriter : public EntryProcessor {
public:
  VuXMLWriter(std::ostream &os);
  virtual bool operator()(const Entry &entry);
  virtual void start();
  virtual void end();
private:
  std::ostream	&os_;
};


class XHTMLWriter : public EntryProcessor {
public:
  XHTMLWriter(std::ostream &os, unsigned int h1);
  virtual bool operator()(const Entry &entry);
  virtual void start();
  virtual void end();
private:
  std::ostream	&os_;
  unsigned int	 h1_;
};


class XHTMLFilesWriter : public EntryProcessor {
public:
  XHTMLFilesWriter(const std::string &dir);
  virtual bool operator()(const Entry &entry);
private:
  std::string    dir_;
  unsigned int   fails_;
};


}
#endif
